package com.pravera.activity_rec_flutter.service

import DatabaseHelper
import android.annotation.SuppressLint
import android.app.NotificationChannel
import android.app.NotificationManager
import android.content.Context
import android.content.Intent
import android.os.Build
import androidx.core.app.JobIntentService
import com.google.android.gms.location.ActivityRecognitionResult
import com.google.gson.Gson
import com.pravera.activity_rec_flutter.Constants
import com.pravera.activity_rec_flutter.errors.ErrorCodes
import com.pravera.activity_rec_flutter.models.ActivityData
import com.pravera.activity_rec_flutter.utils.ActivityRecognitionUtils
import com.pravera.activity_rec_flutter.utils.SharedPreferenceManager
import android.util.Log
import androidx.core.app.NotificationCompat
import androidx.core.app.NotificationManagerCompat
import com.pravera.activity_rec_flutter.R
import kotlin.math.roundToInt

class ActivityRecognitionIntentService : JobIntentService() {

    private lateinit var mPrefs: SharedPreferenceManager
    private val prefRecordedActivity = "prefRecordedActivity"
    private val prefRecordedActivityTime = "prefRecordedActivityTime"
    private val prefStillActivityTime = "prefStillActivityTime"
    private val prefAutoTrackingOnOff = "_prefAutoTrackingOnOff";

    private val oneMinuteInterval = 60000

    private val bikeCO2 = 255
    private val runningCO2 = 265
    private val walkCO2 = 271

    private val perMinBikeDistance = 0.246
    private val perMinBicycleDistance = 0.150
    private val perMinRunningDistance = 0.140
    private val perMinWalkDistance = 0.07435

    private val CHANNEL_ID = "my_channel_id"

    companion object {
        val jsonConverter: Gson = Gson()
        fun enqueueWork(context: Context, intent: Intent) {
            enqueueWork(
                context, ActivityRecognitionIntentService::class.java,
                Constants.ACTIVITY_RECOGNITION_INTENT_SERVICE_JOB_ID, intent
            )
        }
    }

    @SuppressLint("LongLogTag")
    override fun onHandleWork(intent: Intent) {
        val extractedResult = ActivityRecognitionResult.extractResult(intent)
        val probableActivities = extractedResult?.probableActivities
        val mostProbableActivity = probableActivities?.maxByOrNull { it.confidence } ?: return

        mPrefs = SharedPreferenceManager(this)
        val dbHelper = DatabaseHelper(this)

        val activityData = ActivityData(
            ActivityRecognitionUtils.getActivityTypeFromValue(mostProbableActivity.type),
            ActivityRecognitionUtils.getActivityConfidenceFromValue(mostProbableActivity.confidence)
        )

        val autoTrackingStatus = mPrefs.getBoolean(prefAutoTrackingOnOff, false)
        Log.d("ActivityRecognitionIntentService", "autoTrackingStatus $autoTrackingStatus")


        if (autoTrackingStatus && activityData.type != "UNKNOWN") {
            var recordedActivity = mPrefs.getString(prefRecordedActivity, "")
            var recordedActivityTime = mPrefs.getLong(prefRecordedActivityTime, 0L)
            Log.d("ActivityRecognitionIntentService", "activity fetched ${activityData.type}")
            Log.d("ActivityRecognitionIntentService", "recordedActivity =====> ${recordedActivity}")
            Log.d(
                "ActivityRecognitionIntentService",
                "recordedActivityTime =====> ${recordedActivityTime}"
            )

            if (recordedActivity == "") {
                Log.d("ActivityRecognitionIntentService", " =====> recordedActivity == null")
                mPrefs.putString(prefRecordedActivity, activityData.type)
                mPrefs.putLong(prefRecordedActivityTime, System.currentTimeMillis())
            } else {
                Log.d("ActivityRecognitionIntentService", " =====> recordedActivity != null")
                if (activityData.type != recordedActivity) {
                    Log.d(
                        "ActivityRecognitionIntentService",
                        " =====> activityData != recordedActivity"
                    )
                    val stillActivityTime = mPrefs.getLong(prefStillActivityTime, 0)
                    if(activityData.type == "STILL" && stillActivityTime <= oneMinuteInterval){
                        mPrefs.putLong(prefStillActivityTime, System.currentTimeMillis())
                    }else{
                        // Convert milliseconds to minutes
                        val timeDifferenceMillis = System.currentTimeMillis() - recordedActivityTime
                        val timeDifferenceMinutes = timeDifferenceMillis / 60000
                        mPrefs.putString(prefRecordedActivity, activityData.type)
                        mPrefs.putLong(prefRecordedActivityTime, System.currentTimeMillis())

                        when (recordedActivity) {
                            "IN_VEHICLE" -> {
                                val distance = (timeDifferenceMinutes.toInt() * perMinBikeDistance)
                                val co2Vehicle = (distance * bikeCO2 / 1000).roundToInt()
                                val points = (1.2 * distance * co2Vehicle).roundToInt()
                                // val points = timeDifferenceMinutes.toInt() * 0.29

                                sendNotification(recordedActivity, points)

                                Log.d(
                                    "ActivityRecognitionIntentService",
                                    "IN_VEHICLE co2Vehicle -> $co2Vehicle"
                                )
                                Log.d(
                                    "ActivityRecognitionIntentService",
                                    "IN_VEHICLE points -> $points"
                                )
                                val records = dbHelper.getAllActivityPoints()
                                Log.d(
                                    "ActivityRecognitionIntentService",
                                    "IN_VEHICLE records -> ${records.size}"
                                )

                                if (records.isEmpty()) {
                                    dbHelper.addActivityPoints(points.toInt(), 0, 0, 0, 0)
                                } else {
                                    Log.d(
                                        "ActivityRecognitionIntentService",
                                        "IN_VEHICLE first record -> ${records.first()}"
                                    )
                                    dbHelper.editActivityPoints(
                                        records.first().id,
                                        records.first().vehiclePoints + points.toInt(),
                                        records.first().bicyclePoints,
                                        records.first().runningPoints,
                                        records.first().walkingPoints,
                                        records.first().stillPoints,
                                    )
                                }
                            }

                            "ON_BICYCLE" -> {
                                val distance = (timeDifferenceMinutes.toInt() * perMinBikeDistance)
                                val co2Vehicle = (distance * bikeCO2 / 1000).roundToInt()
                                val points = (1.2 * distance * co2Vehicle).roundToInt()
//                            val points = timeDifferenceMinutes.toInt() * 0.50
                                sendNotification(recordedActivity, points)
                                Log.d(
                                    "ActivityRecognitionIntentService",
                                    "ON_BICYCLE points -> $points"
                                )
                                val records = dbHelper.getAllActivityPoints()
                                Log.d(
                                    "ActivityRecognitionIntentService",
                                    "ON_BICYCLE records -> ${records.size}"
                                )
                                if (records.isEmpty()) {
                                    dbHelper.addActivityPoints(0, points.toInt(), 0, 0, 0)
                                } else {
                                    Log.d(
                                        "ActivityRecognitionIntentService",
                                        "ON_BICYCLE first record -> ${records.first()}"
                                    )
                                    dbHelper.editActivityPoints(
                                        records.first().id,
                                        records.first().vehiclePoints,
                                        records.first().bicyclePoints + points.toInt(),
                                        records.first().runningPoints,
                                        records.first().walkingPoints,
                                        records.first().stillPoints,
                                    )
                                }
                            }

                            "RUNNING" -> {
                                val distance = (timeDifferenceMinutes.toInt() * perMinRunningDistance)
                                val co2Running = (distance * runningCO2 / 1000).roundToInt()
                                val points = (1.5 * distance * co2Running).roundToInt()
                                //val points = timeDifferenceMinutes.toInt() * 0.45
                                sendNotification(recordedActivity, points)
                                Log.d("ActivityRecognitionIntentService", "RUNNING points -> $points")
                                val records = dbHelper.getAllActivityPoints()
                                Log.d(
                                    "ActivityRecognitionIntentService",
                                    "RUNNING records -> ${records.size}"
                                )

                                if (records.isEmpty()) {
                                    dbHelper.addActivityPoints(0, 0, points.toInt(), 0, 0)
                                } else {
                                    Log.d(
                                        "ActivityRecognitionIntentService",
                                        "RUNNING first record -> ${records.first()}"
                                    )
                                    dbHelper.editActivityPoints(
                                        records.first().id,
                                        records.first().vehiclePoints,
                                        records.first().bicyclePoints,
                                        records.first().runningPoints + points.toInt(),
                                        records.first().walkingPoints,
                                        records.first().stillPoints,
                                    )
                                }
                            }

                            "WALKING" -> {
                                val distance = (timeDifferenceMinutes.toInt() * perMinRunningDistance)
                                val co2Walking = (distance * runningCO2 / 1000).roundToInt()
                                val points = (1.5 * distance * co2Walking).roundToInt()
                                sendNotification(recordedActivity, points)
                                Log.d(
                                    "ActivityRecognitionIntentService",
                                    "WALKING co2walking -> $co2Walking"
                                )
                                //val points = timeDifferenceMinutes.toInt() * 0.3
                                Log.d("ActivityRecognitionIntentService", "WALKING points -> $points")
                                val records = dbHelper.getAllActivityPoints()
                                Log.d(
                                    "ActivityRecognitionIntentService",
                                    "WALKING records -> ${records.size}"
                                )

                                if (records.isEmpty()) {
                                    dbHelper.addActivityPoints(0, 0, 0, points.toInt(), 0)
                                } else {
                                    Log.d(
                                        "ActivityRecognitionIntentService",
                                        "WALKING first record -> ${records.first()}"
                                    )
                                    dbHelper.editActivityPoints(
                                        records.first().id,
                                        records.first().vehiclePoints,
                                        records.first().bicyclePoints,
                                        records.first().runningPoints,
                                        records.first().walkingPoints + points.toInt(),
                                        records.first().stillPoints,
                                    )
                                }
                            }

                            "STILL" -> {
                                val distance = (timeDifferenceMinutes.toInt() * perMinRunningDistance)
                                val co2Walking = (distance * runningCO2 / 1000).roundToInt()
                                val points = (1.5 * distance * co2Walking).roundToInt()
//                          val points = 0 // per minute points for still
                                val records = dbHelper.getAllActivityPoints()
                                if (records.isEmpty()) {
                                    dbHelper.addActivityPoints(0, 0, 0, 0, 0)
                                } else {
                                    dbHelper.editActivityPoints(
                                        records.first().id,
                                        records.first().vehiclePoints,
                                        records.first().bicyclePoints,
                                        records.first().runningPoints,
                                        records.first().walkingPoints,
                                        records.first().stillPoints,
                                    )
                                }
                            }

                            else -> println("UNKNOWN activity")
                        }
                    }

                }
            }
            dbHelper.addActivity(
                activityData.type,
                activityData.confidence,
                System.currentTimeMillis().toString()
            )
            //dbHelper.getRecords()
        } else {
            Log.d("ActivityRecognitionIntentService", "autoTrackingStatus false or activityData.type == UNKNOWN")
        }


        var prefsKey: String
        var prefsValue: String
        try {
            prefsKey = Constants.ACTIVITY_DATA_PREFS_KEY
            prefsValue = jsonConverter.toJson(activityData)
        } catch (e: Exception) {
            prefsKey = Constants.ACTIVITY_ERROR_PREFS_KEY
            prefsValue = ErrorCodes.ACTIVITY_DATA_ENCODING_FAILED.toString()
        }

        val prefs = getSharedPreferences(
            Constants.ACTIVITY_RECOGNITION_RESULT_PREFS_NAME, Context.MODE_PRIVATE
        ) ?: return
        with(prefs.edit()) {
            putString(prefsKey, prefsValue)
            commit()
        }
    }

    private fun sendNotification(recordedActivity: String?, points: Int?) {
        createNotificationChannel()

        val notificationId = System.currentTimeMillis().toInt()

        val builder = NotificationCompat.Builder(this, CHANNEL_ID)
            .setSmallIcon(applicationIcon)
            .setContentTitle("Activity tracked")
            .setContentText("$recordedActivity activity tracked with $points points")
            .setPriority(NotificationCompat.PRIORITY_HIGH) // Set high priority
            .setCategory(NotificationCompat.CATEGORY_MESSAGE) // Set category
            .setVisibility(NotificationCompat.VISIBILITY_PUBLIC) // Set visibility
            .setAutoCancel(true)
            .setOnlyAlertOnce(true)

        with(NotificationManagerCompat.from(this)) {
            notify(notificationId, builder.build())
        }
    }

    private fun createNotificationChannel() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            val name = "My Channel"
            val descriptionText = "My notification channel"
            val importance = NotificationManager.IMPORTANCE_DEFAULT
            val channel = NotificationChannel(CHANNEL_ID, name, importance).apply {
                description = descriptionText
            }
            val notificationManager: NotificationManager = getSystemService(Context.NOTIFICATION_SERVICE) as NotificationManager
            notificationManager.createNotificationChannel(channel)
        }
    }

    // Function to get the application icon
    private val applicationIcon: Int
        get() = try {
            val appInfo = packageManager.getApplicationInfo(packageName, 0)
            appInfo.icon
        } catch (e: Exception) {
            android.R.drawable.sym_def_app_icon
        }
}
